# -*- coding: utf-8 -*-

import json
import urllib
import urlparse

import sanction
from zope.interface import implements

from .interfaces import *

# A json parser for data returned from a request_token request because sanction
# does not work with a null expires_in
def parser_remove_null_expires_in(data):
    data = json.loads(data)
    if data.get('expires_in', '') is None:
        data.pop('expires_in')
    return data

class OpenstaxAccounts(object):
    implements(IOpenstaxAccounts)

    def __init__(self, server_url, application_id, application_secret,
            application_url):
        resource_url = server_url
        authorize_url = urlparse.urljoin(server_url, '/oauth/authorize')
        token_url = urlparse.urljoin(server_url, '/oauth/token')
        self.redirect_uri = urlparse.urljoin(application_url, '/callback')

        self.sanction_client = sanction.Client(
                auth_endpoint=authorize_url,
                token_endpoint=token_url,
                resource_endpoint=resource_url,
                client_id=application_id,
                client_secret=application_secret)

    def auth_uri(self):
        return self.sanction_client.auth_uri(redirect_uri=self.redirect_uri)

    def request_token_with_code(self, code):
        self.sanction_client.request_token(
                code=code,
                redirect_uri=self.redirect_uri,
                parser=parser_remove_null_expires_in)

    def request_application_token(self):
        self.sanction_client.request_token(
                grant_type='client_credentials',
                parser=parser_remove_null_expires_in)

    def request(self, *args, **kwargs):
        return self.sanction_client.request(*args, **kwargs)


def main(config):
    settings = config.registry.settings
    server_url = settings['openstax_accounts.server_url']
    application_id = settings['openstax_accounts.application_id']
    application_secret = settings['openstax_accounts.application_secret']
    application_url = settings['openstax_accounts.application_url']

    args = (server_url, application_id, application_secret, application_url)

    openstax_accounts = OpenstaxAccounts(*args)
    openstax_accounts.request_application_token()
    config.registry.registerUtility(openstax_accounts, IOpenstaxAccounts)

    config.registry.registerUtility(OpenstaxAccounts(*args), IOpenstaxAccounts,
            'authentication')



import ConfigParser
import functools
import os
import random
import subprocess
import re
import unittest
import urlparse

from selenium import webdriver

def screenshot_on_error(method):
    @functools.wraps(method)
    def wrapper(self, *args, **kwargs):
        try:
            return method(self, *args, **kwargs)
        except:
            self.driver.get_screenshot_as_file('error.png')
            open('error.html', 'w').write(self.driver.page_source.encode('utf-8'))
            raise
    return wrapper

def log(method):
    @functools.wraps(method)
    def wrapper(self, *args, **kwargs):
        result = method(self, *args, **kwargs)
        print repr(result)
        return result
    return wrapper


class FunctionalTests(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.driver = webdriver.Chrome()

        cls.testing_ini = os.getenv('TESTING_INI', 'testing.ini')

        cls.config = ConfigParser.ConfigParser()
        cls.config.read([cls.testing_ini])

        cls.app_url = cls.config.get('app:main', 'openstax_accounts.application_url')
        cls.accounts_url = cls.config.get('app:main', 'openstax_accounts.server_url')

        admin_login = cls.config.get('app:main', 'openstax_accounts.admin_login')
        admin_password = cls.config.get('app:main', 'openstax_accounts.admin_password')

        # login as admin in openstax/accounts
        cls.driver.get(urlparse.urljoin(cls.accounts_url, '/login'))
        cls.class_fill_in('Username', admin_login)
        cls.class_fill_in('Password', admin_password)
        cls.driver.find_element_by_xpath('//button[text()="Sign in"]').click()
        # register our app with openstax/accounts
        cls.driver.get(urlparse.urljoin(cls.accounts_url, '/oauth/applications'))
        cls.driver.find_element_by_link_text('New Application').click()
        cls.class_fill_in('Name', 'pyramid')
        cls.class_fill_in('Redirect uri', urlparse.urljoin(cls.app_url, '/callback'))
        cls.driver.find_element_by_id('application_trusted').click()
        cls.driver.find_element_by_name('commit').click()
        application_id = cls.driver.find_element_by_id('application_id').text
        application_secret = cls.driver.find_element_by_id('secret').text
        cls.driver.quit()

        cls.config.set('app:main', 'openstax_accounts.application_id',
                application_id)
        cls.config.set('app:main', 'openstax_accounts.application_secret',
                application_secret)

        cls.config.write(open(cls.testing_ini, 'w'))

        # start server
        cls.server = subprocess.Popen(['./bin/pserve', cls.testing_ini])

        print 'starting server'
        import time
        time.sleep(3)

    @classmethod
    def tearDownClass(cls):
        cls.server.terminate()

    def fill_in(self, label_text, value):
        label = self.driver.find_element_by_xpath('//label[text()="{}"]'
                .format(label_text))
        input_id = label.get_attribute('for')
        field = self.driver.find_element_by_id(input_id)
        field.send_keys(value)

    class_fill_in = classmethod(fill_in)

    def setUp(self):
        self.driver = webdriver.Chrome()

    def tearDown(self):
        self.driver.quit()

    def follow_link(self, link_text, exact=True):
        if exact:
            self.driver.find_element_by_link_text(link_text).click()
        else:
            self.driver.find_element_by_partial_link_text(link_text).click()

    def generate_username(self, prefix='user'):
        length = 5
        n = '{:0>5d}'.format(random.randint(0, 10 ** length))
        return '{}_{}'.format(prefix, n)

    def page_text(self):
        return re.sub('<[^>]*>', '', self.driver.page_source)

    @screenshot_on_error
    def test_local(self):
        self._test_signup()
        self.tearDown()
        self.setUp()
        self._test_login()

    def _test_signup(self):
        # check that we are not logged in
        self.driver.get(self.app_url)
        self.assertTrue('You are currently not logged in' in self.driver.page_source)
        self.follow_link('Log in')
        # redirected to openstax accounts
        self.follow_link('Sign Up')
        # fill out sign up form
        self.username = self.generate_username()
        self.fill_in('Username', self.username)
        self.fill_in('Password', 'password')
        self.fill_in('Password Again', 'password')
        self.driver.find_element_by_name('commit').click()
        # signed in to openstax accounts
        self.assertTrue('Great, you are signed in' in self.page_text())
        self.follow_link('I have not made', exact=False)
        # profile form
        if 'Complete your profile' in self.page_text():
            self.driver.find_element_by_id('register_i_agree').click()
            self.driver.find_element_by_id('register_submit').click()
        # redirected back to app
        self.assertTrue('You are currently logged in.' in self.page_text())
        # check profile data
        self.follow_link('Profile')
        self.assertTrue('username: {}'.format(self.username) in self.page_text())
        self.assertTrue('id: ' in self.page_text())
        # logout
        self.follow_link('Log out')
        self.assertTrue('You are currently not logged in' in self.page_text())

    def _test_login(self):
        # check that we are not logged in
        self.driver.get(self.app_url)
        self.assertTrue('You are currently not logged in' in self.page_text())
        self.follow_link('Log in')
        # redirected to openstax accounts
        self.fill_in('Username', self.username)
        self.fill_in('Password', 'password')
        self.driver.find_element_by_xpath('//button[text()="Sign in"]').click()
        # redirected back to app
        self.assertTrue('You are currently logged in.' in self.page_text())
        # check profile data
        self.follow_link('Profile')
        self.assertTrue('username: {}'.format(self.username) in self.page_text())
        self.assertTrue('id: ' in self.page_text())
        # logout
        self.follow_link('Log out')
        self.assertTrue('You are currently not logged in' in self.page_text())

    @screenshot_on_error
    def test_facebook(self):
        facebook = dict(self.config.items('test:facebook'))
        # check that we are not logged in
        self.driver.get(self.app_url)
        self.assertTrue('You are currently not logged in' in self.page_text())
        self.follow_link('Log in')
        # redirected to openstax accounts
        self.driver.find_element_by_xpath(
                '//img[@title="Sign in using your Facebook account"]').click()
        # redirected to facebook
        self.fill_in('Email or Phone:', facebook['login'])
        self.fill_in('Password:', facebook['password'])
        self.driver.find_element_by_name('login').click()
        confirm = self.driver.find_elements_by_name('__CONFIRM__')
        if confirm:
            confirm[0].click()
        # redirected back to openstax accounts
        self.assertTrue('Great, you are signed in' in self.page_text())
        self.follow_link('I have not made', exact=False)
        # profile form
        if 'Complete your profile' in self.page_text():
            self.driver.find_element_by_id('register_i_agree').click()
            self.driver.find_element_by_id('register_submit').click()
        # redirected back to app
        self.assertTrue('You are currently logged in.' in self.page_text())
        # check profile data
        self.follow_link('Profile')
        self.assertTrue('username: {}'.format(facebook['username']) in self.page_text())
        self.assertTrue('id: ' in self.page_text())
        # logout
        self.follow_link('Log out')
        self.assertTrue('You are currently not logged in' in self.page_text())

    @screenshot_on_error
    def test_twitter(self):
        twitter = dict(self.config.items('test:twitter'))
        # check that we are not logged in
        self.driver.get(self.app_url)
        self.assertTrue('You are currently not logged in' in self.page_text())
        self.follow_link('Log in')
        # redirected to openstax accounts
        self.driver.find_element_by_xpath(
                '//img[@title="Sign in using your Twitter account"]').click()
        # redirected to twitter
        login = self.driver.find_element_by_id('username_or_email')
        login.send_keys(twitter['login'])
        password = self.driver.find_element_by_id('password')
        password.send_keys(twitter['password'])
        self.driver.find_element_by_id('allow').click()
        # redirected back to openstax accounts
        self.assertTrue('Great, you are signed in' in self.page_text())
        self.follow_link('I have not made', exact=False)
        # profile form
        if 'Complete your profile' in self.page_text():
            self.driver.find_element_by_id('register_i_agree').click()
            self.driver.find_element_by_id('register_submit').click()
        # redirected back to app
        self.assertTrue('You are currently logged in.' in self.page_text())
        # check profile data
        self.follow_link('Profile')
        self.assertTrue('username: {}'.format(twitter['username']) in self.page_text())
        self.assertTrue('id: ' in self.page_text())
        # logout
        self.follow_link('Log out')
        self.assertTrue('You are currently not logged in' in self.page_text())

    @screenshot_on_error
    def test_google(self):
        google = dict(self.config.items('test:google'))
        # check that we are not logged in
        self.driver.get(self.app_url)
        self.assertTrue('You are currently not logged in' in self.page_text())
        self.follow_link('Log in')
        # redirected to openstax accounts
        self.driver.find_element_by_xpath(
                '//img[@title="Sign in using your Google account"]').click()
        # redirected to google
        login = self.driver.find_element_by_id('Email')
        login.send_keys(google['login'])
        password = self.driver.find_element_by_id('Passwd')
        password.send_keys(google['password'])
        self.driver.find_element_by_id('signIn').click()
        # redirected back to openstax accounts
        self.assertTrue('Great, you are signed in' in self.page_text())
        self.follow_link('I have not made', exact=False)
        # profile form
        if 'Complete your profile' in self.page_text():
            self.driver.find_element_by_id('register_i_agree').click()
            self.driver.find_element_by_id('register_submit').click()
        # redirected back to app
        self.assertTrue('You are currently logged in.' in self.page_text())
        # check profile data
        self.follow_link('Profile')
        self.assertTrue('username: {}'.format(google['username']) in self.page_text())
        self.assertTrue('id: ' in self.page_text())
        # logout
        self.follow_link('Log out')
        self.assertTrue('You are currently not logged in' in self.page_text())



# -*- coding: utf-8 -*-

from pyramid.httpexceptions import HTTPFound
from pyramid.interfaces import IAuthenticationPolicy
from pyramid.security import Everyone, Authenticated
from zope.interface import implements

from .interfaces import *


def get_user_from_session(request):
    """Create a helper function for getting the user profile from request.user
    """
    return request.session.get('profile')


class OpenstaxAccountsAuthenticationPolicy(object):
    implements(IAuthenticationPolicy)

    def __init__(self, client, login_path, callback_path):
        self.client = client
        self.login_path = login_path
        self.callback_path = callback_path

    def _login(self, request):
        raise HTTPFound(location=self.client.auth_uri())

    def _callback(self, request):
        code = request.params['code']
        self.client.request_token_with_code(code)

    def authenticated_userid(self, request):
        if request.path == self.login_path:
            return self._login(request)
        if request.path == self.callback_path:
            self._callback(request)
            me = self.client.request('/api/users/me.json')
            request.session.update({
                'profile': me,
                'username': me.get('username'),
                })
            request.session.changed()
            return me.get('username')
        return self.unauthenticated_userid(request)

    def unauthenticated_userid(self, request):
        return request.session.get('username')

    def effective_principals(self, request):
        groups = [Everyone]
        if self.authenticated_userid(request):
            groups.append(Authenticated)
        return groups

    def remember(self, request, principal, **kw):
        pass

    def forget(self, request):
        request.session.clear()


def main(config):
    config.add_request_method(get_user_from_session, 'user', reify=True)
    settings = config.registry.settings
    config.set_authentication_policy(OpenstaxAccountsAuthenticationPolicy(
        client=config.registry.getUtility(IOpenstaxAccounts, 'authentication'),
        login_path=settings['openstax_accounts.login_path'],
        callback_path=settings['openstax_accounts.callback_path'],
        ))



from setuptools import setup

setup(
        name='openstax-accounts',
        version='0.2',
        description='An example pyramid app that connects to openstax/accounts',
        long_description=open('README.rst').read(),
        author='Karen Chan',
        author_email='karen@karen-chan.com',
        url='http://github.com/karenc/openstax-accounts',
        packages=['openstax_accounts'],
        install_requires=(
            'PasteDeploy',
            'pyramid',
            'sanction',
            'waitress',
            ),
        tests_require=(
            'selenium',
            ),
        test_suite='openstax_accounts.tests',
        entry_points={
            'paste.app_factory': [
                'main = openstax_accounts.example:main',
                ],
            },
        zip_safe=False,
        )
